/*
 ================================================================================
 DCF77 Analyzer / Clock
 ================================================================================
 This sketch is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This sketch is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
 The C++ code is far from optimized because I myself am an Arduino and C++ novice.
 But even after learning some more now, I want to keep the code simpel and readable.
 That is why I maybe over-documented the code to help understand what's going on.

 HELP WANTED: 
 I'm not experienced enough to get to the point where the second-tick
 of the Real Time Clock display is exactly in sync with the DCF pulse.
 Now there's a small time lag.
 If you have a solution, please let me know! 
 

 // ***************Test serial print of leapyear*************************
 
 Erik de Ruiter
 2014-2016
 
 May 2014 First version
 March 2016 - big overhaul...
 
 
 Version 1.72
 - Option: Use a cheap Ebay PIR detector to shut off selectable display's when no activity is detected. 
   The switch off delay can be set by the user to prevent the display shutting of if a person
   is not moving but the display should be on.
 - Now the display Night shut-down can be disabled by making both values 'POWERSAVINGOFFTIME'
   and 'POWERSAVINGONTIME' zero. 
 - Fixed temperature display not shutting off at powersave mode.  
 - errorCounter display did not reset every hour so that's fixed

 Version 1.71
 - User option to reset temperature min/max memory at midnight

 Version 1.7:
 - The resolution of the temperature display is improved: from 0.5 to 0.1 degrees Celsius
   Because of the time the DS18B20 sensor needs to convert the temperature and to keep the code clean, 
   the temperature display is updates once per minute.
 - Parity check routine optimized. 
 - More reliable check for bad DCF data, preventing RTC update with invalid data.
 - EoB error now clears inner LED ring as it should.
 - The DCF OK LED now displays the condition of the DCF signal more reliably. Turns off immediately if an error occurs
   and only turns ON when all 3 parity bits are OK.

 Version 1.6:
 - Changed temperature function to only calculate once per minute. Got strange errors before the change because
   I used a delay of 100ms to give the DS18B20 sensor time to calculate the temperature. But the delay function is
   a very bad idea in most c++ code so I finally got rid of it.

 Version 1.5:
 - Complete overhaul of the scanSignal function and the rest of the code! My first attempt worked but could be improved...
 - The rPW and rPT led's did not work as I intended so that is corrected now.
 - The End of Buffer error check routine does work now as it should.
 - I incorporated a Parity check of the incoming DCF signal. In the signal 3 Parity bits are sent so now these are
   checked and only if all three are OK, the received time information is accepted, the display is updated and the RTC synced.
   if desired, you can attach 3 extra dual-color LED's (Common Cathode) to see if each of the 3 Parity bits are OK or Failed.
 - I made wiring (or changing the wiring) much easier I think by putting all the PIN config in one easy to read table
 - As long as you use 1 DS18B20 temp. sensor, I edited the code so you no longer need to figure out the address of the I2C device.
 - Big clean-up of the code...
 - Powersaving by shutting off the displays (the clock remains functioning as normal)
   can now be configured somewhat easier by editing two variables POWERSAVINGONTIME and POWERSAVINGOFFTIME.
 - changed some variable names:
   - Maxim instances 'lc' and 'lc1' are now MaximCC and MaximCA
   - Display description MaximDcfTime is now MaximTemperature 
   - DCF77SOUNDPIN is now BUZZERPIN
 - LED/Display test after power up now build in
 
 ***********************************************************************************************
 Brett Oliver 05/06/19 DCF77 Analyzer Mk2
 v2
 v2_3 Extra transistor added for steady Superfilter LEDs
 v2_4 Intensity on LED test lowred line 452 and removed int ldrout = A2; as not needed
 v2_5 Blank super filter display LEDs when no motion detected by setting LDR value to 0 line 1316 added delay to LED test to allow superfilter LED test to complete
 v2_6 remove buzzer control input use direct on off switch on buzzer instead
 v2_7 changed sound duration of 1 to make difference more audable
 v2_8 reverse 7 segment module for bit number etc Bit nunumber error count and Bit value working for small 7 segment modules note error on line 1654 shift to be added to 4 lines below
 v2_9 sorted error with period time and pulse width display errors
 v3_0 sort intenity error between large and small 7 segment displays
 v3_1 sorting 3rd LED matrix order
 v3_2 sorting 3rd LED Matrix pt 2
 v3_3 sorting 3rd LED Matrix pt 3 and corrected LED test on MatrixLED module 2
 v3_4 stopped error counter reset every hour now resets every time correct code is received
 v3_5 adjust outer and inner ring LEDs to match matrix PCB pins also changed LEDtest as I have 60 LEDs on inner and outer rings
 v3_6 added LEDTEST_DELAY_LED_RING2 to control delay in 3rd matrix led test (DCF status day of week etc LEDs. Timings are slower for these LEDs as they are spread around the display
 v3.8 added JQ6500 sound modules
 v3_9 added chimes to JQ6500
 v4_0 working version with modified JQ6500 library
 v4_2 added chime timer/24hr chime and chime volume
 v4_3 changed chime timer
******* To do  
 blank un cols see line 1346
 blank MaximLeds unused cols (display2)
 add short test chime for vol up and down less than 1 second


 
 
 Brett Oliver 27/06/18
 Added radar module to replace PIR timer search for PIRtimer set to 300 seconds (+ PIR or Radar module timeout). If movement detected in this period timer starts again.
 
 
 Brett Oliver 22/04/2017
 Version 61
 Fixed error on leapYear causing Leap year LED to stay on  if (leapYear = 1) changed to if (leapYear == 1)
 LED display brightness on startup changed to 10 so LED Display test can be seen in daylight
 Unused LEDs blanked every second
 
 Brett Oliver 25/05/2016
 Version 60
 Converted my v34 to Eric's v1.72 to work on my clock hardware.
 Parity LED's x6 added to MaximLEDs
 CA display & temperature display removed
 Time is displayed on a CC display with colon's added with a mask
 This version uses the unmodified LEDControl library as no CA displays are used
 No chime or power saving option used on this version
 PIR detection is always on, delay and sensitivity controlled by the PIR module
 Eriks LED test added to all LEDs on Maxim display and order of LEDs test modified to suit my hardware
 Remote control of;
 1 Udo Klein super filter indicator LEDs
 2 DCF77 buzzer On/Off
 3 Super Filter reset
 4 Main Arduino Reset
 Remote control signal received LED on Maxim LED display
 Note 1. I have a Udo Klein super filter on the DCF77 input to this clock. This can be switched On & Off
 Note 2. This clock displays the raw DCF77 signal and converts it to show date and day of week etc as CET.
 Time is shown as GMT so an hour before midnight GMT the date and Day of week will change early as per CET.
 
 Thanks to Erik for sharing his code and clock design!
 ***********************************************************************************************
 
  Short description:
   
  Power On:
    After power-on, first a LED test is performed. The LED's and displays lite up sequentially to keep the power consumption low.
    Then the clock starts receiving DCF pulses and when a Minute Mark (2 seconds gap) is detected, the Minute Marker LED is lit
    and the buffer counter is reset. The inner LED ring now will show the incoming DCF pulses which are also stored in the buffer.
    At 3 moments during reception of data the parity DCF bits are checked to see if the data is valid.

  Valid data received:
    When, at the end of the minute, after the Minute Mark is detected (BF (Buffer Full) LED is lit), all three parity bits are OK
    ('DCF OK' LED is lit), the buffer information is used to extract time and date information. 
    Then the RTC clock is updated ('RTC Synced' LED is lit) and the inner LED ring information is copied to the outer LED ring. 
    The time, date and week display, day LED, summer/wintertime and leap year LED information is updated with the new time information.

  No valid data:
    When one or more of the parity bits are not OK because of a noisy signal, receiving of DCF information is continued but
    will not be used to update the RTC, display's and LED's. The outer LED ring, 'RTC synced' and 'DCF OK' LED's will be reset. 
    Time, date, week, day LED, summer/wintertime LED and leap year LED are not affected and keep displaying the last received valid values.
    The 'Period Time' and/or 'Period With' error LED's will indicate the error(s) and the error counter display is updated. 
    Every hour, the error display will bet set to zero. 
    The EoB, End of Buffer LED is lit when more DCF pulses are received before the Minute Mark is detected due to a noisy signal.
    (When a minute Mark is detected we should have no more than 58 bits/pulses) 
    After the detection of the Minute Marker, a new cycle is started.
 
  Temperature:
    At the 30 second mark, the temperature display will show the High and Low values of the past period after the last reset.
  
  Chime:
    At the beginning of each hour, the Chime (if connected) will sound. 
    At night time, a time set by the user in the code itself, the chime is disabled.

  Power saving, two options:
    1. NIGHT SHUT OFF
       At times set by the user, the displays are shutt off at night and turned on in the morning.
       Look at the POWERSAVINGOFFTIME and POWERSAVINGONTIME variables. 
       Check the function <turnDisplaysOff> to select which displays you want to shut off at night.
    2. PIR SENSOR
       Connect a PIR sensor and activate the PIR option POWERSAVE_BY_PIR and the the delay at PIR_DELAY_TIME.
       Every time the PIR detector senses movement, a minute counter is reset but if no movement is detected
       longer than the PIR_DELAY_TIME, the displays are shut off. 
       When movement occurs, the displays immediately switch on. 
    Note: as said before, the clock will function normally while the displays are shut off. 
    The only thing is you can't see it... ;)

  DCF beep:
    With a switch, connected to pin BUZZERPIN, you can hear the received DCF bits coming in. 
    The tone duration is equivalent to pulse width of the DCF bits, so either 100 or 200 ms.
  
  Miscelleanous:
    When the RTC battery is empty or a connection fault is detected, the RTC Error LED is lit.




 CREDITS:
 I learned a lot from the work of Matthias Dalheimer and Thijs Elenbaas who made their own DCF77 decoders.
 Without their work I would not have known where to start.
 I ended up writing my own code (using bits and pieces of their ideas) so I could understand what is happening...
 My code is far from efficient or advanced but it does work and I know what is going on.
 
 * A big Thank You to Brett Oliver and Joop Tap for pointing out some errors!

 Interesting websites:

 - Brett Oliver         : http://www.brettoliver.org.uk/
 - Joop Tap             : http://www.jooptap.nl
 - Thijs Ellenbaas      : http://thijs.elenbaas.net/2012/04/arduino-dcf77-radio-clock-receiver-hardware-2/
 - Mathias Dalheimer    : https://github.com/roddi/DCF77-Arduino/blob/master/DCF77Servoclock/DCF77.h
 - DCF77 wikipedia      : https://en.wikipedia.org/wiki/DCF77
 - Much more DCF77 info : http://www.picbasic.nl/indexes_uk.htm

 - My Flickr website    : https://www.flickr.com/photos/edr1924/albums
 - My Github website    : https://github.com/deruiter
 - The Instructables website for this clock: soon!

 */



//----------------------------------------------------------------------------------------------------------
// Libraries
//----------------------------------------------------------------------------------------------------------

// Arduino (new) Time library .................................... http://www.pjrc.com/teensy/td_libs_Time.html
#include <Time.h>

// Enable this line if using Arduino Uno, Mega, etc.
#include <Wire.h>

// a basic DS1307 library that returns time as a time_t .......... http://www.pjrc.com/teensy/td_libs_DS1307RTC.html
#include <DS1307RTC.h>

// Maxim 7219 displays library ................................... http://playground.arduino.cc/Main/LEDMatrix
// !!! NOTE: you must use a special version of the Ledcontrol.h library to get Common Anode support 
// because the Maxim chip is normally only suitable for common CATHODE displays!
// Brett standard lirary used in this clock
#include <LedControl.h>

//SPI interface library .......................................... http://arduino.cc/en/Reference/SPI
#include <SPI.h>

//JQ9500 sound module
//#include <Arduino.h>
#include <SoftwareSerial.h>
#include <JQ6500_Serial.h>
// Create the mp3 module object, 
//   Arduino Pin 7 is connected to TX of the JQ6500
//   Arduino Pin 8 is connected to one end of a  1k resistor, 
//     the other end of the 1k resistor is connected to RX of the JQ6500
//   If your Arduino is 3v3 powered, you can omit the 1k series resistor
JQ6500_Serial mp3(7,8);
//JQ6500_Serial mp3(0,1);

//----------------------------------------------------------------------------------------------------------
// Arduino UNO Pin connections in an easy to read table
//
                              // Pin  0 - input  - Rx - used for programming/communication with PC
                              // Pin  1 - output - Tx - used for programming/communication with PC
                              
#define DCF77PIN            2 // Pin  2 - input  - DCF signal from antenna pcb. Must be pin 2 or 3 because of interrupt!
//#define          3   // INPUT: from superfilter
//#define BUZZERPIN           4 // Pin  4 - input  - SWITCH - turn on/off DCF77 'beep' piezo buzzer / ON = HIGH, OFF = LOW
#define MAXIMCCLD          10 // Pin 10 - output - CS/LOAD - Maxim Common Cathode 7 segment displays
#define MAXIMCCCLK         11 // Pin 11 - output - CLOCK   - Maxim Common Cathode 7 segment displays
#define MAXIMCCDATA        12 // Pin 12 - output - DATA    - Maxim Common Cathode 7 segment displays
#define BUZZER             A1 // Pin A1 - output - Piezo buzzer for DCF77 'beep' (to '+' of the buzzer)
#define LEDintensity        9 // Pin 9 - output - to Transistor 1 to control brightness of Flashing Superfilter LEDs
#define LEDintensitya        6 // Pin 6 - output - to Transistor 2 to control brightness of Steady Superfilter LEDs
// USED for DS1307 RTC        // Pin A4 - I2C DATA  - connect to Real Time Clock pcb
// USED for DS1307 RTC        // Pin A5 - I2C CLOCK - connect to Real Time Clock pcb



//----------------------------------------------------------------------------------------------------------
// Maxim 7219 Matrix Display initialization
//----------------------------------------------------------------------------------------------------------
/*
 clearDisplay(int addr) ........................................ clears the selected display
 MaximCC.shutdown(int addr, boolean) ................................ wake up the MAX72XX from power-saving mode (true = sleep, false = awake)
 MaximCC.setIntensity(int addr, value) .............................. set a medium brightness for the Leds (0=min - 15=max)
 MaximCC.setLed(int addr, int row, int col, boolean state) .......... switch on the led in row, column. remember that indices start at 0!
 MaximCC.setRow(int addr, int row, byte value) ...................... this function takes 3 arguments. example: MaximCC.setRow(0,2,B10110000);
 MaximCC.setColumn(int addr, int col, byte value) ................... this function takes 3 arguments. example: MaximCC.setColumn(0,5,B00001111);
 MaximCC.setDigit(int addr, int digit, byte value, boolean dp) ...... this function takes an argument of type byte and prints the corresponding digit on the specified column.
                                                                 The range of valid values runs from 0..15. All values between 0..9 are printed as digits,
                                                                 values between 10..15 are printed as their hexadecimal equivalent
 MaximCC.setChar(int addr, int digit, char value, boolean dp) ....... will display: 0 1 2 3 4 5 6 7 8 9 A B C D E F H L P; - . , _ <SPACE> (the blank or space char)
 
 ***** Please set the number of devices you have *****
 But the maximum default of 8 MAX72XX wil also work.
 LedConrol(DATAIN, CLOCK, CS/LOAD, NUMBER OF MAXIM CHIPS)
 */

// MaximCC is for the Maxim Common CATHODE displays
LedControl MaximCC = LedControl(MAXIMCCDATA, MAXIMCCCLK, MAXIMCCLD, 7); // Define pins for Maxim 72xx and how many 72xx we use

                             
//----------------------------------------------------------------------------------------------------------
// User settings, variable and array definitions
//----------------------------------------------------------------------------------------------------------

// The value below is not a PIN number but a value to set how many times the 'Lower volume' input on the sound board is activated
// so that way the volume of the sound board can be lowered after power up, if desired.
//#define SPEAKERVOLUME  12            

// Choose if you want a test of all LED's and Displays after a startup
// '1' = Yes, '0' = No
#define PERFORM_LED_TEST 1
// Delay between each 7 segment display in ms
#define LEDTEST_DELAY_DISPLAYS 1000
// Delay between each LED in the LED ring inner and outer inms
#define LEDTEST_DELAY_LED_RING 25
//Delay between each LED in the 3rd LED matrix info LED's in ms
#define LEDTEST_DELAY_LED_RING2 100 
// Choose if you want to configure the DS18B20 temperature sensor ONCE to the highest resolution.
// this is needed after using the sensor for the first time. After running the software
// with this setting ON one time, shut it off.
// '1' = ON, '0' = OFF
//#define CONFIGURE_DS18B20 0

// define power saving display OFF and ON time
// values are in 'Hour' format
// ONLY the displays are shut off at power saving time, the clock remains active.
// To disable this feature, make both values zero
#define POWERSAVINGOFFTIME 0  // displays are activated
#define POWERSAVINGONTIME  0 // displays are shutt off

// User option to reset temperature min/max memory at midnight
// '1' = Reset at midnight, '0' = Only manual reset
//#define TEMPRESET_MIDNIGHT 1

// User option: activate the displays only when there is activity in the room
// '1' = ON, '0' = OFF
//#define POWERSAVE_BY_PIR 1
// delay in MINUTES to wait after no detection before shutting off the displays 
//#define PIR_DELAY_TIME 5 

//-------------------------------------------------------------------------------
// define miscellaneous parameters
#define DS1307_I2C_ADDRESS  0x68      // define the RTC I2C address
#define DCF_INTERRUPT       0         // Interrupt number associated with pin

// definition of Maxim 7219 display number wiring sequence
// first Maxim 7219 in wiring 'daisychain' must be '0', next '1' etc.
// COMMON CATHODE DISPLAYS
#define MaximLedRingInner   0
#define MaximLedRingOuter   1
#define MaximPeriodPulse    6
#define MaximBufferBitError 5
#define MaximWeek           7 // Brett not used
#define MaximDate           4 
#define MaximRtcTime        3
#define MaximLeds           2
/*
// COMMON ANODE DISPLAYS
#define MaximTemperature    0
*/
// definition of display brighness levels
// Brett use intensity if different intensity levels required on Dot matrix and 7 seg displays
//#define BrightnessMaximLedRingInner     intensity2
//#define BrightnessMaximLedRingOuter     intensity2
//#define BrightnessMaximLeds             intensity2
#define BrightnessMaximLedRingInner     intensity4
#define BrightnessMaximLedRingOuter     intensity4
#define BrightnessMaximLeds             intensity3
#define BrightnessMaximRtcTime          intensity
#define BrightnessMaximDate             intensity
#define BrightnessMaximBufferBitError   intensity2
#define BrightnessMaximPeriodPulse      intensity2
//#define BrightnessMaximWeek             3 // Brett week display not used


// definition of Status/Error Led's. We use division and modulo to seperate Row/Col values
// to lit a LED you need a ROW and COLUMN value.
// so, for example: BFLed (with value 14) divided by 10 results in the value '1' (row)
// and BFLed % (Modulo) 10 results in the value '4' (column)
// Modulo explained: http://www.cprogramming.com/tutorial/modulus.html

/* Row/Col LED numbers of Maxim 7219 chip
 LED            Row Col  MK2
 -------------- --- ---  ----------------------------------------------------------------
 Sunday          0   1    0   0
 Monday          0   2    1   0
 Tuesday         0   3    2   0
 Wednesday       0   4    3   0
 Thursday        0   5    4   0
 Friday          0   6    5   0
 Saturday        0   7    6   0
 Synced          5   1    4   1   ON when RTC is set by the received DCF time
 RTCError        5   7    3   1   ON when there is no response from the RTC (connection / battery empty)
 SummerTime      7   5    0   1   ON when Central European Summer Time is set
 WinterTime      7   4    1   1   ON when Central European Time is set
 LeapYear        7   6    2   1   ON when we have a Leap Year... Duh!
 BF              5   6    0   2   Flashes ON shortly when the DCF buffer if full and can be processed
 EoM             5   5    0   3   Flashed ON shortly when the Minute Mark is detected BEFORE the DCF buffer is filled (after power up or other errors)
 EoB             5   4    7   2   Flashes ON shortly when too many pulses are received before the Minute Marker (noisy signal)
 rPW             5   3    5   1   Flashes ON shortly when the time between two pulse periods is out of set limits
 rPT             5   2    6   1   Flashes ON shortly when the pulse time is out of set limits
 DCFOk           5   0    1   3   ON when whe have a received a full minute of valid DCF data
 SpkLED      7    0
 FilterLED   7    1
 Remote R3   7    2
 LEDs        7    3
 Remote      7    4

 Chime           6   7    Not usedON when the Chime is enabled. If it's night time or wh switch the Chime off, this LED should be OFF
 LEDP1Pass       6   0    6   2   ---
 LEDP1Fail       6   1    5   2   Parity Check LED's, use if desired. 
 LEDP2Pass       6   2    4   2   WHILE receiving the incoming bits, at 3 moments, you can see if the Parity bits are OK or FAILED... 
 LEDP2Fail       6   3    3   2   I use 3 dual color 3mm LED's for this purpose but you can also only use the 'PASS' LED's)
 LEDP3Pass       6   4    2   2
 LEDP3Fail       6   5    1   2   ---  

*/
//#define SignalLED      70 Brett deleted MK2
#define SyncedLed      41
#define RTCError       31
#define SummerTimeLed  01
#define WinterTimeLed  11
#define LeapYearLed    21
#define BFLed          02
#define EoMLed         03
#define EoBLed         72
#define rPWLed         51
#define rPTLed         61
#define DCFOKLed       13
// #define ChimeLed       67 Brett deleted MK2
#define LEDP1Pass      62
#define LEDP1Fail      52
#define LEDP2Pass      42
#define LEDP2Fail      32
#define LEDP3Pass      22
#define LEDP3Fail      12
// #define SpkLED         71 Brett deleted MK2
//#define FilterLED      72 Brett Deleted Mk2
//#define LEDs           73
//#define RemoteLED      74
#define intensityLED   intensityDisp

// Pulse flanks
static unsigned long flankTime    = 0;
static unsigned long leadingEdge  = 0;
static unsigned long trailingEdge = 0;
unsigned long previousLeadingEdge = 0;

// used in <Int0handler>
volatile unsigned int DCFSignalState = 0; // interrupt variables ALWAYS need volatile qualifier!!

// used in <loop>
int previousSecond      = 0;
int previousSignalState = 0;
int twoSecondPulse      = 5; // IC pin 11

//Chimes

int chimeAlways         = 3; // Turns On Off chime timer
int chimevolUp          = 17;
int chimevolDn          = 16;
int chimevolume         = 0;
int chimeFile           = 0;
int chimetestup           = 0;// used to test chime when vol + is pressed and let go
int chimetestdn           = 0;// used to test chime when vol - is pressed and let go
int chimestart            = 1; // enables chimes when timer on
int chimeon               = 0;

// DCF Buffers and indicators
static int DCFbitBuffer[59]; // here, the received DCFbits are stored
const int bitValue[] = {1, 2, 4, 8, 10, 20, 40, 80}; // these are the decimal values of the received DCFbits

// only after start on a new minute, display received bits on inner LED ring
boolean MinuteMarkerFlag = false;
int bufferPosition       = 0;
int previousMinute       = 0;
int previousHour         = 0;

// used to check if switch state is changed



// variables to check if DCF bits are vald
bool dcfValidSignal  = false;
int dcfP1counter     = 0;
int dcfP2counter     = 0;
int dcfP3counter     = 0;
int dcfParityCheckP1 = 0;
int dcfParityCheckP2 = 0;
int dcfParityCheckP3 = 0;

int GMThour = 0;

// display intensity Brett
int intensity = 5; // Intensity of MAXIM 7 digit LEDs

int intensity2 = 5; // intensity of MAXIM small 7 segment LEDs

int intensity3 = 5; // intensity of MAXIM Matrix LEDs

int intensity4 = 5; // intensity of inner & outer ring LEDs LEDs


int intensityDisp = 10;
int ldr = A0; // LDR connected to analogue A0 pin
//int ldrout = A2; //output to transistor Q1 to control brightness of super filter LEDs
unsigned int ldrValue; // LDR value

int filterSw = 0;

// PIR detector for blanking displays Brett
int PIRval = 0; // infrared value
int PIR = 13; // set pin 13 as input from PIR
int PIRlock = 0; // when PIR is triggerred this will hold it on until the next quarter hour and 1 second.
int PIRtimer = 1;


// Remote control switch toggles Brett
//int remote1val; // current value of remote sw no1
//int remote1prev = 0; // previous value of remote sw no1
//int remote1in =5;  // remote control input 4 is pin 5 (pin 11 on IC)
/*
int remote2val; // current value of remote sw no2
int remote2prev = 0; // previous value of remote sw no2
int remote2in =6;  // remote control input 3 is pin 6 (pin 12 on IC)

int remote3val; // current value of remote sw no3
int remote3prev = 0; // previous value of remote sw no3
int remote3in = 7;  // remote control input 2 is pin 7 (pin 13 on IC)
int LEDctrl = 0;
int remote3out = 8;


int remote4val = 0; // current value of remote sw no4
int remote4prev = 0; // previous value of remote sw no4
int remote4in =8;  // remote control input 1 is pin 8 (pin 14 on IC)

 // Remote control monitor LED lights when any remote button pressed
int RemoteLEDpin = 9;
int Remotepin = 0;
*/
// DCF77 Signal LED
//int SignalLEDpin = 4;
//int SignalLEDval = 0;



// dcf variables to store decoded DCF time in
int dcfMinute  = 0;
int dcfHour    = 0;
int dcfDay     = 0;
int dcfWeekDay = 0;
int dcfMonth   = 0;
int dcfYear    = 0;
int dcfDST     = 0;
int leapYear   = 0;

// variables used to store weeknumber and daynumer values
int dayNumber;
int weekNumber;

// error counter variable
int errorCounter        = 0;
boolean errorCondition  = false;

// miscelleanous variables
boolean daytimeChange   = true;
boolean dayTime         = false;        
int dcf77SoundSwitch    = 0;


//==============================================================================
// SETUP
//==============================================================================
void setup()
{
  // initialize Serial communication
  //Serial.begin(115200);
  Serial.begin(9600);

  // initialize PIN connections
  pinMode(DCF77PIN,      INPUT);
  
 
 pinMode(chimeAlways,      INPUT);//used to set chime timer/chime 24/7  (24/7 LOW Timer High)
  pinMode(chimevolUp,      INPUT); 
  pinMode(chimevolDn ,      INPUT); 




  
  //pinMode(TEMPRESETPIN,  INPUT);
  //pinMode(BUZZERPIN,     INPUT);
   
   pinMode(PIR,INPUT);           // Brett PIR detector for blanking displays
    pinMode(twoSecondPulse, OUTPUT); // IC pin 11
//  pinMode(PIRDETECTORPIN,INPUT);
//  pinMode(CHIMEPIN,      OUTPUT);
//  pinMode(SPEAKERVOLPIN, OUTPUT);
//  pinMode(remote1in, INPUT);  //Brett remote sw 
//  pinMode(remote2in, INPUT);  //Brett remote sw
//  pinMode(remote3in, INPUT);  //Brett remote sw
 // pinMode(remote3out, OUTPUT);  //Brett remote sw outputs on pin 14 on ic when button3 pressed (Superfilter LEDs on
//  pinMode(remote4in, INPUT);  //Brett remote sw
//  pinMode(RemoteLEDpin, INPUT); //Brett remote ctrl monitor led
//  pinMode(SignalLEDpin, INPUT); //Brett DCF77 Signal monitor LED
  //pinMode(ldrout, OUTPUT); // //output to transistor Q1 to control brightness of super filter LEDs
  //pinMode(LEDintensity, OUTPUT);
  
 
  // Initialize variables and LED displays
  initialize();

  // Initialize DCF77 pulse interrupt on pin DCF_INTERRUPT, looking for a change of the signal,
  // so either rising or falling edge pulses will trigger the interrupt handler and
  // execute the int0handler function.
  attachInterrupt(DCF_INTERRUPT, int0handler, CHANGE);

  // Initialize RTC and set as SyncProvider.
  // Later RTC will be synced with DCF time
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  // check if RTC has set the system time
  if (timeStatus() != timeSet)
  { // Unable to sync with the RTC - activate RTCError LED
    MaximCC.setLed(MaximLeds, RTCError / 10, RTCError % 10, true);
  } 
  else {
    // RTC has set the system time - dim RTCError LED
    MaximCC.setLed(MaximLeds, RTCError / 10, RTCError % 10, false);
  }

   /*
   Serial.print ("Chimestart "); 
    Serial.print (chimestart);
     Serial.print (" Chimeon "); 
    Serial.println (chimeon);
*/
     chimevolume = 15;
  // use for test purposes and/or setting the RTC time manually
  //#####################################################################################
  // setTime(5, 42, 40, 31, 12, 13);
  // RTC.set(now());
//#####################################################################################
 
  // check if a LED test is needed 
  
  if(PERFORM_LED_TEST == 1)
  {
    // do a LED test
    ledTest();
  }
  else
  {
    // if not doing a LED test, we need to wait a bit for the DS18B20 sensor to get ready
    delay(750);
  }
  


  // activate errorCounter display after LED test
  ledDisplay(MaximBufferBitError, "R", 0);//Brett was R
 //ledDisplay(MaximBufferBitError, "L", 6);//Brett was R 0
  chimestart = 1;// set timer ready for startup

 //JQ9500 sound module
  mp3.begin(9600);
  mp3.reset();
  mp3.setVolume(30);
  mp3.setLoopMode(MP3_LOOP_NONE);

}


//==============================================================================
// LOOP
//==============================================================================
void loop()
{
  // check first if pulse direction is changed (rising or falling)
  // else we would keep evaluating the same pulse
  if (DCFSignalState != previousSignalState)
  {
    // 'reset' state of variable
    previousSignalState = DCFSignalState;

    // evaluate incoming pulse
    scanSignal();
  }

  // check if switches are changed and act upon it
  checkSwitches();
   //quarterHourChime();
 
 //hourChime();



/*
// Brett DCF77 signal monitor
        
        SignalLEDval = digitalRead(SignalLEDpin); // reads OP from remote sw output
        
        if (SignalLEDval == 0)
        {
          
          MaximCC.setLed(MaximLeds, SignalLED / 10, SignalLED % 10, false);
        }
        else
                 
          MaximCC.setLed(MaximLeds, SignalLED / 10, SignalLED % 10, true);
          */
        /*  
          Serial.print("SignalLEDval ");
      Serial.println(SignalLEDval);
     */   
        // // Brett DCF77 signal monitor
 

     
        // Brett light remote LED on operation of any remote button
        
  //      Remotepin = digitalRead(RemoteLEDpin); // reads OP from remote sw output
      /*  
        if (Remotepin == 0)
        {
          
          MaximCC.setLed(MaximLeds, RemoteLED / 10, RemoteLED % 10, false);
        }
        else
                 
          MaximCC.setLed(MaximLeds, RemoteLED / 10, RemoteLED % 10, true);
        
          Serial.print("Remotepin ");
      Serial.println(Remotepin);
     */   
        // Brett light remote LED on operation of any remote button
 
 


  
  // execute tasks that must happen only once every second, minute or hour
  //----------------------------------------------------------------------------
  tasksEverySecond();
  tasksEveryMinute();
  tasksEveryHour();
}






//================================================================================================================
//
// Function name : processDcfBit
// called from   : <scanSignal>
//
// Purpose       : Evaluates the signal as it is received. Decides whether we received a "1" or a "0"
//                 and perform checks to see if the pulse timing is within limits
// Parameters    : none
// Return value  : none
//
//================================================================================================================
/*
       pulse                 pulse
       width                 width
       |- -|               |--   --|           |----- END OF MINUTE marker:2000ms -----|
        ___                 _______             ___                                     ___                 _______
       | 0 |               |   1   |           | 0 |                                   | 0 |               |   1   |
       |   |               |       |           |   |                                   |   |               |       |
       |   |               |       |           |   |                                   |   |               |       |
 ______|   |_______________|       |___________|   |___________________________________|   |_______________|       |__ _ _ _
       ^   ^               ^       ^           ^   ^               ^                   ^   ^               ^       ^
       1000 2100           2000    2200        3000 3100         NO PULSE              5000 5100           6000    6200         << example millis() value
                                                                 = end of Minute indication
       ^                   ^                   ^                                       ^                   ^
       DCFbit# 56          DCFbit# 57          DCFbit# 58                               DCFbit# 0           DCFbit# 1  etc...   << DCF bit received
       
       ^                   ^        ^
       previous            leading  trailing
       leading edge        edge     edge
       
       ^   ^
       flanktime (rising or falling)
 
 */

void scanSignal()
{
  //--------------------------------------------------------------------
  // Check for Rising-Edge signal and perform checks
  //--------------------------------------------------------------------
  if (DCFSignalState == 1) 
  {
    // store Rising-Edge Time to check later if the time between two pulses is valid
    leadingEdge = millis();
    // not much to do now so exit.
    return;
  }

  //--------------------------------------------------------------------
  // Check for Falling-Edge signal and perform checks
  //--------------------------------------------------------------------
  
  if (DCFSignalState == 0)
  {
    // store Trailing-Edge Time to check later if the Pulse Width is valid
    trailingEdge = millis();

    // display period width time on "L"eft side of the 8 digit Maxim 72xx LED display
  ledDisplay(MaximPeriodPulse, "L", (leadingEdge - previousLeadingEdge));//BRETT CORRECT FOR SMALL 7 SEGMENT DISPLAY
    // ledDisplay(MaximPeriodPulse, "R", (leadingEdge - previousLeadingEdge));
    // display pulse width time on the "R"ight side of the 8 digit Maxim 72xx LED display
  //  ledDisplay(MaximPeriodPulse, "R", (trailingEdge - leadingEdge));//Brett changed
    ledDisplay(MaximPeriodPulse, "R", (trailingEdge - leadingEdge));//BRETT CORRECT FOR SMALL 7 SEGMENT DISPLAY

    //--------------------------------------------------------------------------------
    // Check PERIOD TIME
    //--------------------------------------------------------------------------------
    // If this flank UP is detected quickly after previous flank UP this is an incorrect
    // Period Time (should be 1000ms -or 2000ms after second 58-) that we shall reject
    if ((leadingEdge - previousLeadingEdge) < 900)
    {
      // rPW - ERROR: Periode Time (rising flank to rising flank) time is too short -> REJECTED
      error(rPWLed);
      errorCondition = true;
    }
    //--------------------------------------------------------------------------------
    // CHECK PULSE TIME
    //--------------------------------------------------------------------------------
    // If the detected pulse is too short it will be an incorrect pulse that we shall reject
    // should be 100 and 200 ms ideally
    if (((trailingEdge - leadingEdge) < 70) || ((trailingEdge - leadingEdge) > 230))
    {
      //rPT - ERROR: Pulse Width too short or too long -> REJECTED
      error(rPTLed);
      errorCondition = true;
    }

    // if we had an error return and start over
    if (errorCondition == true)
    {
      errorCondition = false;
      // although we have an error, store current rising edge time to compare at the next Rising-Edge.
      previousLeadingEdge = leadingEdge;
      return;
    }


    //--------------------------------------------------------------------
    // no errors found so now we can continue
    //--------------------------------------------------------------------
   
    // first we turn any error Led's OFF
    MaximCC.setLed(MaximLeds, rPWLed / 10, rPWLed % 10, false);
    MaximCC.setLed(MaximLeds, rPTLed / 10, rPTLed % 10, false);
    MaximCC.setLed(MaximLeds, BFLed  / 10, BFLed  % 10, false);
    MaximCC.setLed(MaximLeds, EoBLed / 10, EoBLed % 10, false);
    MaximCC.setLed(MaximLeds, EoMLed / 10, EoMLed % 10, false);
    
    // END OF MINUTE check, looking for a gap of approx. 2000ms
    if ( leadingEdge - previousLeadingEdge > 1900 && leadingEdge - previousLeadingEdge < 2100)
    {
     // end of minute detected:
      finalizeBuffer();
    }

    // refresh previousLeadingEdge time with the new leading edge time
    previousLeadingEdge = leadingEdge;

    //--------------------------------------------------------------------------------
    // process DCF bits
    //--------------------------------------------------------------------------------
    // distinguish between long and short pulses
    if (trailingEdge - leadingEdge < 170)
    {
      // call processDcfBit function and sent it the value '0'
      processDcfBit(0);
      
      buzzer(100);
     
    }
    else
    {
      // call processDcfBit function and sent it the value '1'
      processDcfBit(1);
      
     buzzer(300);//Brett changed from 200 to make 1 and 0 more noticable
    }
  } // if (DCFSignalState == 0)
} // void scanSignal();

//================================================================================================================
//
// Function name : processDcfBit
// called from   : <scanSignal>
//
// Purpose       : after reception of one good DCF bit, do some checks and save it in the DCFbitBuffer array
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void processDcfBit(int dcfBit)
{
  //--------------------------------------------------------------------
  // display values on the 7 segment displays
  //--------------------------------------------------------------------
  // display bufferPosition, digits 7,6
 // MaximCC.setChar(MaximBufferBitError, 0, bufferPosition / 10, false);// Brett sets order of Bit number
 // MaximCC.setChar(MaximBufferBitError, 1, bufferPosition % 10, false);// Brett sets order of Bit number
    MaximCC.setChar(MaximBufferBitError, 7, bufferPosition / 10, false);// Brett was 1 correct for small 7 seg module
    MaximCC.setChar(MaximBufferBitError, 6, bufferPosition % 10, false);// Brett was 0 correct for small 7 seg module
 
  
  // display received DCFbit, digit 4
  //MaximCC.setChar(MaximBufferBitError, 3, dcfBit, false); //Brett set position of Bit Value from right
  MaximCC.setChar(MaximBufferBitError, 4, dcfBit, false);// correct for small 7 seg modules
  //--------------------------------------------------------------------
  // display incoming DCF bits on inner LED ring
  //--------------------------------------------------------------------
  // only if we have valid DCF data or after an Minute Mark (EoM) signal 
  // activate the inner LED ring and diplay incoming data
  if (dcfValidSignal == true || MinuteMarkerFlag == true) 
  {
    // display received bits on inner LED ring
   // MaximCC.setLed(MaximLedRingInner, bufferPosition / 8, bufferPosition % 8, dcfBit);
    MaximCC.setLed(MaximLedRingInner, bufferPosition % 8, bufferPosition / 8, dcfBit);//Brett adjusted to match dot matrix module PCB
  }

  //--------------------------------------------------------------------
  //   // Fill DCFbitBuffer array with DCFbit 
  //--------------------------------------------------------------------
  DCFbitBuffer[bufferPosition] = dcfBit;
 
  //--------------------------------------------------------------------
  // Parity check
  //--------------------------------------------------------------------
  // DURING reception of the DCF bits, calculate and display the results of the DCF parity check.
  //
  // There is a Parity bit for the minutes, the hours and for the date.
  // DCF77 works with EVEN parity, this works as follows:
  // The hours for example have 6 bits plus a paritybit. The bits with value 1 are add up including the paritybit,
  // the result must be an even number. If there is a bit wrong received, a 0 is as 1, or a 1 is as 0 received, 
  // then the result is uneven.  source: http://www.picbasic.nl/frameload_uk.htm?http://www.picbasic.nl/info_dcf77_uk.htm

  if (bufferPosition == 0)
  {
    // reset the parity LED's
    MaximCC.setLed(MaximLeds, LEDP1Pass / 10, LEDP1Pass % 10, false);
    MaximCC.setLed(MaximLeds, LEDP1Fail / 10, LEDP1Fail % 10, false);
    MaximCC.setLed(MaximLeds, LEDP2Pass / 10, LEDP2Pass % 10, false);
    MaximCC.setLed(MaximLeds, LEDP2Fail / 10, LEDP2Fail % 10, false);
    MaximCC.setLed(MaximLeds, LEDP3Pass / 10, LEDP3Pass % 10, false);
    MaximCC.setLed(MaximLeds, LEDP3Fail / 10, LEDP3Fail % 10, false);
    // reset variables
    dcfP1counter = 0;
    dcfP2counter = 0;
    dcfP3counter = 0;
    dcfParityCheckP1 = 0;
    dcfParityCheckP2 = 0;
    dcfParityCheckP3 = 0;
  }

  // ----------------------------------------
  // First parity check: minute bits
  // ----------------------------------------
  if (bufferPosition == 28)
  {
    for(int i = 21; i <= 27; i++)
    {
      // count the number of bits with the value '1'
      dcfP1counter += DCFbitBuffer[i];
    }

    // perform P1 parity check. Parity is OK if the sum is an EVEN value
    if((DCFbitBuffer[28] + dcfP1counter) % 2 == 0)
    {
      // Parity1 PASS LED ON
      MaximCC.setLed(MaximLeds, LEDP1Pass / 10, LEDP1Pass % 10, true);
      // Parity P1 PASS
      dcfParityCheckP1 = 1;
    }
    else 
    {
      // Parity1 FAIL LED ON
      MaximCC.setLed(MaximLeds, LEDP1Fail / 10, LEDP1Fail % 10, true);
      // we have no valid data!
      dcfValidSignal = false;
      // Turn DCF OK LED OFF
      MaximCC.setLed(MaximLeds, DCFOKLed / 10, DCFOKLed % 10, false);
    }
  }

  // ----------------------------------------
  // Second parity check: hour bits
  // ----------------------------------------
  if (bufferPosition == 35)
  {
    for(int i = 29; i <= 34; i++)
    {
      dcfP2counter += DCFbitBuffer[i];
    }

    // perform P2 parity check. Parity is OK if the sum is an EVEN value
    if((DCFbitBuffer[35] + dcfP2counter) % 2 == 0)
    {
      // Parity2 PASS LED ON
      MaximCC.setLed(MaximLeds, LEDP2Pass / 10, LEDP2Pass % 10, true);
      // Parity P2 PASS
      dcfParityCheckP2 = 1;
     }
    else 
    {
      // Parity2 FAIL LED ON
      MaximCC.setLed(MaximLeds, LEDP2Fail / 10, LEDP2Fail % 10, true);
      // we have no valid data!
      dcfValidSignal = false;
      // Turn DCF OK LED OFF
      MaximCC.setLed(MaximLeds, DCFOKLed / 10, DCFOKLed % 10, false);
    }
  }

  // ----------------------------------------
  // Third parity check: date bits
  // ----------------------------------------
  if (bufferPosition == 58)
  {
    for(int i = 36; i <= 57; i++)
    {
      dcfP3counter += DCFbitBuffer[i];
    }
    // perform P3 parity check. Parity is OK if the sum is an EVEN value
    (DCFbitBuffer[58] + dcfP3counter) % 2 == 0 ? dcfParityCheckP3 = 1 : dcfParityCheckP3 = 0;

      // Turn Parity2 'PASS' or 'FAIL' LED ON
    if(dcfParityCheckP3 == 1)
    {
      // Parity2 PASS LED ON
      MaximCC.setLed(MaximLeds, LEDP3Pass / 10, LEDP3Pass % 10, true);
      // Parity P3 PASS
      dcfParityCheckP3 = 1;
    }
    else 
    {
      // Parity2 FAIL LED ON
      MaximCC.setLed(MaximLeds, LEDP3Fail / 10, LEDP3Fail % 10, true);
      // we have no valid data!
      dcfValidSignal = false;
      // Turn DCF OK LED OFF
      MaximCC.setLed(MaximLeds, DCFOKLed / 10, DCFOKLed % 10, false);
    }
  
    // ----------------------------------------
    // finally, check all Parity bits
    // ----------------------------------------
    dcfParityCheckP1 + dcfParityCheckP2 + dcfParityCheckP3 == 3 ? dcfValidSignal = true : dcfValidSignal = false;
  }

  //--------------------------------------------------------------------
  // before continuing with the next bit, increment counter
  //--------------------------------------------------------------------
  bufferPosition++;


  //--------------------------------------------------------------------
  // check if we have not received too many pulses?
  //--------------------------------------------------------------------
  if (bufferPosition > 59)
  {
    // End of Buffer (EoB) ERROR - we have received more pulses before reaching
    // the 2 second 'gap' signalling the end of the minute. 
    //This error may be due to a noisy signal giving addition peaks/dcfBits
    // So clear both DCFbit displays and start again.

    // Reset buffer counter
    bufferPosition = 0;
    // clear inner LED ring
    MaximCC.clearDisplay(MaximLedRingInner);
    // turn EoB Error LED ON
    error(EoBLed);
    // exit
    return;
  }

  //--------------------------------------------------------------------
  // everything OK so we wait for next incoming DCFbit
  //--------------------------------------------------------------------
}

//================================================================================================================
//
// Function name : finalizeBuffer
// called from   : <scanSignal>
//
// Purpose       : Process the succesfully received DCF data of one minute
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void finalizeBuffer(void) 
{
  //--------------------------------------------------------------------
  // We are here because of the detected 2 second 'gap'.
  // Now check if it correspondends with the buffer counter
  // 'bufferPosition' which should be value 59
  //--------------------------------------------------------------------
  if (bufferPosition == 59 && dcfValidSignal == true)
  {
    // bufferPosition == 59 so turn Buffer Full LED ON
    MaximCC.setLed(MaximLeds, BFLed / 10, BFLed % 10, true);

    // Turn DCF OK LED ON
    MaximCC.setLed(MaximLeds, DCFOKLed / 10, DCFOKLed % 10, true);

    // Reset inner LED ring (incoming time information)
    MaximCC.clearDisplay(MaximLedRingInner);

    // copy 'contents' of inner LED ring to the outer LED ring (current time information)
    for (int i = 0; i < 59; i++) 
    {
     // MaximCC.setLed(MaximLedRingOuter, i / 8, i % 8, DCFbitBuffer[i]);
      MaximCC.setLed(MaximLedRingOuter, i % 8, i / 8, DCFbitBuffer[i]); //Brett set to match dot matrix PCB wiring
    }

    // process buffer and extract data sync the time with the RTC
    decodeBufferContents();
  
    // set Arduino time and after that set RTC time
    setTime(dcfHour, dcfMinute, 0, dcfDay, dcfMonth, dcfYear);
    RTC.set(now());
  
    // activate Synced LED
    MaximCC.setLed(MaximLeds, SyncedLed / 10, SyncedLed % 10, true);

    // Reset running buffer
    bufferPosition   = 0;

    // Reset DCFbitBuffer array, positions 0-58 (=59 bits)
    for (int i = 0; i < 59; i++) {
      DCFbitBuffer[i] = 0;
    }

    // reset flag
    MinuteMarkerFlag = false;

    //Brett reset error count when signal decoded OK
    //--------------------------------------------------------------------
    errorCounter = 0;
    // update error counter display
  ledDisplay(MaximBufferBitError, "R", errorCounter);//Brett was R
   // ledDisplay(MaximBufferBitError, "L", errorCounter);//Brett was R


    //Brett reset error count when signal decoded OK
    
  } // if (bufferPosition == 59)


  //--------------------------------------------------------------------
  // The buffer is not yet filled although the 2 second 'gap' was detected.
  // Can be result of a noisy signal, starting in middle of receiving data etc.
  // Turn End of Message LED ON
  //--------------------------------------------------------------------
  else
  {
    MaximCC.setLed(MaximLeds, EoMLed / 10, EoMLed % 10, true);

    // Clear displays
    MaximCC.clearDisplay(MaximLedRingInner);
    MaximCC.clearDisplay(MaximLedRingOuter);

    // Reset running buffer and start afresh. Now we are in sync with the incoming data
    bufferPosition   = 0;

    // Reset DCFbitBuffer array, positions 0-58 (=59 bits)
    for (int i = 0; i < 59; i++) 
    {
      DCFbitBuffer[i] = 0;
    }

    // set flag so we can display incoming pulsed on the inner LED ring.
    MinuteMarkerFlag = true;
  }
}

//================================================================================================================
//
// Function name : decodeBufferContents
// called from   : <finalizeBuffer>
//
// Purpose       : Evaluates the information stored in the buffer. 
//                 This is where the DCF77 signal is decoded to time and date information
// Parameters    : none
// Return value  : none
//
//================================================================================================================
 
void decodeBufferContents(void) 
{
  // Buffer is full and ready to be decoded
  dcfMinute  = bitDecode(21, 27);
  dcfHour    = bitDecode(29, 34);
  dcfDay     = bitDecode(36, 41);
  dcfWeekDay = bitDecode(42, 44);
  dcfMonth   = bitDecode(45, 49);
  dcfYear    = bitDecode(50, 57);

  //call function to calculate day of year and weeknumber
  dayWeekNumber(dcfYear, dcfMonth, dcfDay, dcfWeekDay);

  // Get value of Summertime DCFbit. '1' = Summertime, '0' = wintertime
  dcfDST     = bitDecode(17, 17);

  // determine Leap Year
  leapYear   = calculateLeapYear(dcfYear);
}

//================================================================================================================
//
// bitDecode
//
// called from <processBuffer>
//================================================================================================================
int bitDecode (int bitStart, int bitEnd) 
{
  // reset 'bitValue-array' counter
  int i = 0;
  int value = 0;

  // process bitrange bitStart > bitEnd
  while (bitStart <= bitEnd)
  {
    // check if DCFbit in buffer is '1', discard when '0'
    if (DCFbitBuffer[bitStart] == 1) {
      // DCFbit in buffer == 1 so append its corresponding value to the variable 'value'
      value = value + bitValue[i];
    }
    // increment 'bitValue-array' counter
    i++;
    // increment bit-range counter
    bitStart++;
  }
  return value;
}

//================================================================================================================
//
// Function name : tasksEverySecond
// called from   : <loop>
//
// Purpose       : perform tasks that must happen once every SECOND
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void tasksEverySecond()
{
  // check if time is changed
  if (second() != previousSecond)
  {
    // 'reset' variable state
    previousSecond = second();

// Blanking unused LEDs

for (int i = 0; i < 8; i++) {
// turns off LEDs on col 1 and 4
//MaximCC.setLed(MaximLeds, 1, i, false); 
//MaximCC.setLed(MaximLeds, 4, i, false);
}
// turns off individual unused LEDs
//MaximCC.setLed(MaximLeds, 0, 7, false);
//MaximCC.setLed(MaximLeds, 6, 6, false);
//MaximCC.setLed(MaximLeds, 6, 7, false);
 MaximCC.setChar(MaximBufferBitError, 5, " ", true); // blank character between bit value and bit number
 MaximCC.setChar(MaximBufferBitError, 5, ".", false); // blank character between bit value and bit number

 
// Set chime timer or chime on 24/7
// If Chime Timer switch is on (HIGH)
//Once triggerred chime stays on until deactivated by time set below to deactivate
// On initial power up time is checked just before every 1/4 to see if chime should be ON
  if ( digitalRead(chimeAlways) == HIGH && hour() >= 6 && (minute() == 14 || minute() == 29 || minute() == 44 || minute() == 59)  && chimestart == 1) //Note CET/CEST used to set chime timer/chime 24/7  (24/7 LOW & Timer High)
  {
   
   chimestart = 0;
   chimeon = 1;
  
   Serial.print ("chimeAlways "); 
    Serial.println ("TIMER");
    Serial.print ("Chimestart "); 
    Serial.println (chimestart);
     Serial.print ("Chimeon "); 
    Serial.println (chimeon);
    
  }

  else if ( digitalRead(chimeAlways) == LOW)
   
   {
   chimeon = 1;
    Serial.print ("chimeAlways "); 
   Serial.println ("24/7");
    Serial.print ("Chimestart "); 
    Serial.println (chimestart);
     Serial.print ("Chimeon "); 
    Serial.println (chimeon);
   }


// If Chime Timer switch is ON HIGh but time is out of range. Once de-activasted chime stays off untill time is in range angaim
 if ( digitalRead(chimeAlways) == HIGH && hour() >= 0 && hour()<6 && minute()== 1 && chimestart == 0) //Note CET/CEST used to set chime timer/chime 24/7  (24/7 LOW & Timer High)
  {
   
   chimestart = 1;
   chimeon = 0;
   Serial.print ("chimeAlways "); 
    Serial.println ("TIMER OFF ");
    Serial.print ("Chimestart "); 
    Serial.println (chimestart);
     Serial.print ("Chimeon "); 
    Serial.println (chimeon);
    
  }

 // sound hour chimes and 1/4 hour chimes if chimeon set to 1 from above 2 if statements  
if ( chimeon == 1 )
{
   quarterHourChime();
    hourChime();
    
}
  
  


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   
// Change chime volume.
//Hold vol up or volume down for over a second to increment or decrement volume
// Plays a test tone when button is released

//Volume up

 if( digitalRead(chimevolUp) == LOW)
 {
   ++chimevolume;
  ++chimevolume;
   ++chimevolume;
   
   //max chime volume is 30 
   if(chimevolume>30)
   {
    chimevolume = 30;
   }
   // end max chime vol is 30
   
   mp3.setVolume(chimevolume);
  chimetestup = 1;
Serial.print ("Chime Volume "); 
 Serial.println (chimevolume);
//  Serial.print ("Chimetestup"); 
 //   Serial.println (chimetestup);
 }
 
// Test tone played on release of volume up button
if(chimetestup == 1 && digitalRead(chimevolUp) == HIGH)
{
  
mp3.playFileByIndexNumber(17);// plays short test chime to test volume
chimetestup = 0;
//Serial.print ("Chimetestup"); 
 //   Serial.println (chimetestup);
}


//Volume down
 
 if( digitalRead(chimevolDn) == LOW)
 {
  --chimevolume;
   --chimevolume;
   --chimevolume;
  
    //min chime volume is 0 
    if(chimevolume<0)
   {
    chimevolume = 0;
   }
  //end min chime volume is 0 

  
  mp3.setVolume(chimevolume);
 chimetestdn = 1;

  Serial.print ("Chime Volume "); 
    Serial.println (chimevolume);
 //   Serial.print ("Chimetestdn"); 
 //   Serial.println (chimetestdn);
 }
// Test tone played on release of volume down button
if( chimetestdn == 1 && digitalRead(chimevolDn) == HIGH )
{
mp3.playFileByIndexNumber(17);// plays short chime to test volume
chimetestdn = 0;
//Serial.print ("Chimetestdn"); 
//    Serial.println (chimetestdn);
}

 

 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  





  // Trigger hard wired JQ9500 every other second to give tick tock sound
  if ( (second() % 2) == 0) 
  { 
    
  digitalWrite(twoSecondPulse, HIGH);





  
  }

 else
 {
  digitalWrite(twoSecondPulse, LOW);
 }

   

    //display the Real Time Clock Time
    displayRtcTime();
    motiondetection(); //Brett check motion detection every second and blank/show displays
      // Brett check LDR value once per seconds and display value
    intensityValue();

    // blank Superfilter LEDs when PIR detects no motion
    if (PIRlock == 0)
    {
    ldrValue = 0; // read value of LDR
    }
    else
    {
     ldrValue = analogRead(ldr); // read value of LDR
    }
    
     
   //Serial.print ("LDR Value ");
 // Serial.println (ldrValue, DEC);
  //Serial.print (" ");
  //  intensityValue(); // converts LDR value into intensity value 1 to 16 and also value foe MaximLED display
  //  Serial.print ("Intensity "); 
  //  Serial.print (intensity);
  //  Serial.println (" ");
  //  Serial.print ("IntensityDisp "); 
  //  Serial.print (intensityDisp);
  //  Serial.println (" ");
    
    
    
    // End Brett check LDR value once per seconds and display value
 
  // Brett blank Rows 1,4 and 6 on MaximLED display
  //MaximCC.setRow(2,1,B0000000);  // blank maximled row 1
 // MaximCC.setRow(2,4,B0000000);  // blank maximled row 4
 // MaximCC.setRow(2,6,B0000000);  // blank maximled row 6
 
 
    // display 'HI' and 'LO' temperature on specific moments
    switch (second())
    {
    
    case 2:
      // hourly chime output: DEACTIVATE
 //     digitalWrite(CHIMEPIN, LOW);
      break;
  
    }// switch
  }// (second() != previousSecond)
}// void tasksEverySecond()


//================================================================================================================
//
// Function name : tasksEveryMinute
// called from   : <loop>
//
// Purpose       : perform tasks that must happen once every MINUTE
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void tasksEveryMinute()
{  
  // display date, week LED's, week nr etc. if time is changed
  if (minute() != previousMinute)
  {
    // 'reset' state of variable
    previousMinute = minute();

     Serial.print ("Time "); 
    Serial.print (hour());
    Serial.print (":");
    Serial.println (minute());

    Serial.print ("Chimestart "); 
    Serial.print (chimestart);
     Serial.print (" Chimeon "); 
    Serial.println (chimeon);

    // display date, week LED's, week nr etc.
    if (dcfValidSignal == true)
    {
      displayData(); 
    }

    // increase PIR delay counter
  //  pirTimer++;

  }
}

//================================================================================================================
//
// Function name : tasksEveryHour
// called from   : <loop>
//
// Purpose       : perform tasks that must happen once every HOUR
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void tasksEveryHour()
{ 
  if (hour() != previousHour)
  {
    // 'reset' variable state
    previousHour = hour();

  //Brett error counter now reset after every good decode
    //--------------------------------------------------------------------
    // reset error counter and display every hour
    //--------------------------------------------------------------------
 /*
    errorCounter = 0;
    // update error counter display
  ledDisplay(MaximBufferBitError, "R", errorCounter);//Brett was R
   // ledDisplay(MaximBufferBitError, "L", errorCounter);//Brett was R
    */
//end Brett error counter now reset after every good decode
    //---------------------------------------------------------------------
    // Power saving function, shutting the displays off at night
    //---------------------------------------------------------------------

    // First, check if the night shut-down function is activated by the user
    // simply by adding up both value. If '0', the user has disabled the function
    if(POWERSAVINGOFFTIME != 0 && POWERSAVINGONTIME != 0)
    { 
      // check whether it is Day- or Nighttime
      if (hour() >= POWERSAVINGOFFTIME && hour() <= POWERSAVINGONTIME)
      {
        // ----------------------------------------
        // it's DAYTIME so activate the displays
        // ----------------------------------------
        
        // this is used to chime only if it is daytime...
        dayTime = 1; 

         // test variable because daytime routine is needed only once
        if (daytimeChange == 1)
        {
          // 'reset' variable state
          daytimeChange = 0;

          // activate SELECTED displays and status LED's
          turnDisplaysOn();
        }
      }
      else
      {
        // no chime at night...
        dayTime = 0; 

        // ----------------------------------------
        // it's NIGHTTIME so time to deactivate displays
        // ----------------------------------------
        // test variable because nighttime routine is needed only once
        if (daytimeChange == 0) 
        {
          // 'reset' variable state
          daytimeChange = 1; 

          // deactivate all the displays and status LED's
          turnDisplaysOff();

        }// if (daytimeChange == 0)
      }// else
    }// if(POWERSAVINGOFFTIME + POWERSAVINGONTIME != 0)
  }// if (dcfHour != previousHour)
}// void tasksEveryHour()


//================================================================================================================
//
// Function name : buzzer
// called from   : <scanSignal>
//
// Purpose       : generate 'beep' sound
// Parameters    : duration in ms
// Return value  : none
//
//================================================================================================================

void buzzer(int duration)
{
  tone(BUZZER, 1500, duration);
}

//================================================================================================================
//
// Function name : initialize
// called from   : <Setup>
//
// Purpose       : initialize variables and displays after power-up
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void initialize(void)
{
  //---------------------------------------------------
  // Initialize Variables
  //---------------------------------------------------
  leadingEdge           = 0;
  trailingEdge          = 0;
  previousLeadingEdge   = 0;
  bufferPosition        = 0;

//  digitalWrite(CHIMEPIN, HIGH); // Set Chimepin to default state

  // Reset DCFbitBuffer array, positions 0-58 (=59 bits)
  for (int i = 0; i < 59; i++) {
    DCFbitBuffer[i] = 0;
  }


  //---------------------------------------------------
  // Initialize Maxim 72xx 7 segment displays
  //---------------------------------------------------
  // Maxim Common Cathode displays
  for (int i = 0; i < 7; i++)
  {
    // display wake up
    MaximCC.shutdown(i, false);
    // clear display
    MaximCC.clearDisplay(i);
  }
 
  //---------------------------------------------------
  // Set brightness of Maxim 72xx 7 segment displays
  //---------------------------------------------------
  // Maxim Common Cathode displays
  MaximCC.setIntensity(MaximLedRingOuter,   BrightnessMaximLedRingOuter);
  MaximCC.setIntensity(MaximLedRingInner,   BrightnessMaximLedRingInner);
  MaximCC.setIntensity(MaximPeriodPulse,    BrightnessMaximPeriodPulse);
 // MaximCC.setIntensity(MaximWeek,           BrightnessMaximWeek);
  MaximCC.setIntensity(MaximDate,           BrightnessMaximDate);
  MaximCC.setIntensity(MaximRtcTime,        BrightnessMaximRtcTime);
  MaximCC.setIntensity(MaximLeds,           BrightnessMaximLeds);
  MaximCC.setIntensity(MaximBufferBitError, BrightnessMaximBufferBitError);
 
}

//================================================================================================================
//
// Function name : int0handler
// called from   : 
//
// Purpose       : when a rising or falling edge is detected on pin 2, this function is called
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void int0handler()
{
  DCFSignalState = digitalRead(DCF77PIN);
}

//================================================================================================================
//
// Function name : turnDisplaysOn
// called from   : <tasksEveryHour> and <checkPIR>
//
// Purpose       : turn ON selected 7 segment displays and LED's
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void turnDisplaysOn()
{
  // activate SELECTED displays and status LED's
  MaximCC.shutdown(MaximRtcTime, false);
  MaximCC.shutdown(MaximDate, false);
  MaximCC.shutdown(MaximWeek, false);
  MaximCC.shutdown(MaximLeds, false);
  MaximCC.shutdown(MaximLedRingInner, false);
  MaximCC.shutdown(MaximLedRingOuter, false);
  MaximCC.shutdown(MaximPeriodPulse, false);
  MaximCC.shutdown(MaximBufferBitError, false);
  
}

//================================================================================================================
//
// Function name : turnDisplaysOff
// called from   : <tasksEveryHour> and <checkPIR>
//
// Purpose       : turn OFF selected 7 segment displays and LED's
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void turnDisplaysOff()
{
  // below you can select which display's need to be shut down for the night
  // In this case, the time display remains ON

  //MaximCC.shutdown(MaximRtcTime, true);
  MaximCC.shutdown(MaximDate, true); 
  MaximCC.shutdown(MaximWeek, true);
  MaximCC.shutdown(MaximLeds, true);
  MaximCC.shutdown(MaximLedRingInner, true);
  MaximCC.shutdown(MaximLedRingOuter, true);
  MaximCC.shutdown(MaximPeriodPulse, true);
  MaximCC.shutdown(MaximBufferBitError, true);
 
}

//================================================================================================================
//
// Function name : ledDisplay
// called from   : <processBuffer>
//
// Purpose       : display a value on a selected 7 segment display
// Parameters    : display 'number 'addr', 'value', "L"eft or "R"ight side.none
// Return value  : none
//
//================================================================================================================

// example: ledDisplay( MaximBufferBitError, "R", errorCounter )
// so display the 'error counter' value on the RIGHT side of the 8 digit
// 7 segment display number 3

void ledDisplay(int addr, String leftOrRight, int value)

{
  int ones;
  int tens;
  int hundreds;
  int thousands;
  int shift;

  //break down value in seperate digits for ones, tens, etc
  int v = value; // 'value' is needed later so copy it in 'v'
  ones = v % 10;
  v = v / 10;
  tens = v % 10;
  v = v / 10;
  hundreds = v % 10;
  thousands = v / 10;

  //Select which side of the 8 digit display to be used
 // (leftOrRight == "L" ? shift = 7 : shift = 4);
 (leftOrRight == "L" ? shift = 4 : shift = 0); //Brett when L selected shift value is added to L address. See line 726 where L & R are assigned to Period Time & Pulse Width

  //Now print the number digit by digit
  //preceding zero's are removed with tenary operator by 'printing' a space character.
  // Brett works with small 7 segment display modules
 
   MaximCC.setChar(addr, 3 + shift , ((value < 1000) ? ' ' : thousands), false);
  MaximCC.setChar(addr, 2 + shift , ((value < 100) ? ' ' : hundreds), false);
  MaximCC.setChar(addr, 1 + shift , ((value < 10) ? ' ' : tens), false);
  MaximCC.setChar(addr, 0 + shift , ones, false);
  
 
/*
 * works with large 7 segment display modules
 MaximCC.setChar(addr, 0 + shift, ((value < 1000) ? ' ' : thousands), false);
  MaximCC.setChar(addr, 1 + shift, ((value < 100) ? ' ' : hundreds), false);
  MaximCC.setChar(addr, 2 + shift, ((value < 10) ? ' ' : tens), false);
  MaximCC.setChar(addr, 3 + shift, ones, false);
*/
  
}



//================================================================================================================
//
// Function name : checkSwitches
// called from   : <loop>
//
// Purpose       : check if the temperature reset button button is pressed
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void checkSwitches(void)
{

 
}

//================================================================================================================
//
// Function name : checkPIR
// called from   : <loop>
//
// Purpose       : check for PIR detector activity to shut off or activate the displays to save power 
// Parameters    : none
// Return value  : none
//
//================================================================================================================

//================================================================================================================
//
// Function name : error
// called from   : <scanSignal>
//
// Purpose       : turn error LED ON, clear LED ring's and increase error counter display
// Parameters    : error LED to turn on 
// Return value  : none
//
//================================================================================================================

void error(int errorLed)
{
  // no valid data
  dcfValidSignal = false;

  // turn 'dcfValidSignal = false on'
  MaximCC.setLed(MaximLeds, errorLed / 10, errorLed % 10, true);

  // clear Led's/displays because of error condition
  MaximCC.setLed(MaximLeds, DCFOKLed / 10, DCFOKLed % 10, false);
  MaximCC.setLed(MaximLeds, SyncedLed / 10, SyncedLed % 10, false);
  MaximCC.clearDisplay(MaximLedRingOuter); // clear display

  // increase errorCounter and display errorCount
  errorCounter++;
  ledDisplay(MaximBufferBitError, "R", errorCounter);//Brett was L changed 18/06/19 to correct error count errors
   //ledDisplay(MaximBufferBitError, "L", errorCounter);//Brett was R
  return;
}

//================================================================================================================
//
// Function name : dayWeekNumber
// called from   : <decodeBufferContents>
//
// Purpose       : calculate the WEEK number according to ISO standard, see comments in the ARCHIVE below
// Parameters    : dcfYear, dcfMonth, dcfDay, dcfWeekDay 
// Return value  : weekNumber
//
//================================================================================================================
//Code from: http://forum.arduino.cc/index.php/topic,44476.0.html

int dayWeekNumber(int y, int m, int d, int w)
{
  // Number of days at the beginning of the month in a normal (not leap) year.
  int days[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}; 
  
  // Start to calculate the number of days of the first two months
  if (m == 1 || m == 2) 
  {
    // for any type of year we calculate the number of days for January or february
    dayNumber = days[(m - 1)] + d;
  }

  // now calculate for the other months
  // first, check for a leap year
  else if ((y % 4 == 0 && y % 100 != 0) ||  y % 400 == 0) 
  { 
    // we have a leap year, so calculate in the same way but adding one day
    dayNumber = days[(m - 1)] + d + 1;
  }
 
  else
  {
    //no leap year, calculate in the normal way, such as January or February
    dayNumber = days[(m - 1)] + d;
  }
  
  // Now start to calculate Week number
  if (w == 0) 
  {
    //if it is sunday (time library returns 0)
    weekNumber = (dayNumber - 7 + 10) / 7;
  }
  
  else
  {
    // for the other days of week
    weekNumber = (dayNumber - w + 10) / 7;
  }

  // finished! return with the week number as an INT value
  return weekNumber;
}


//================================================================================================================
//
// Function name : calculateLeapYear
// called from   : <decodeBufferContents>
//
// Purpose       : determine if a given year is a leap year
// Parameters    : year - the year to test
// Return value  : '1' if the year is a leap year, '0' otherwise
//
//================================================================================================================

int calculateLeapYear(int year)
{
  if ( (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0) ) 
  {
    return 1;
  } 
  else 
  {
    return 0;
  }
}

//================================================================================================================
//
// Function name : displayData
// called from   : <tasksEveryMinute>
//
// Purpose       : display LED's: day of week; status; Winter/Summer time;
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void displayData(void)
{
  // display Day of Week on LED's
  // first, clear all the 'Day' Led's (row '0') before displaying new value
  for (int i = 0; i < 7; i++) 
  {
   // MaximCC.setLed(MaximLeds, 0, i, false);
      MaximCC.setLed(MaximLeds, i, 0, false);
  }

  // Maxim 7219 LED's are numbered from zero so sunday '7' must be changed to '0'
 // MaximCC.setLed(MaximLeds, 0, ((dcfWeekDay == 7) ? 0 : dcfWeekDay), true);
  MaximCC.setLed(MaximLeds, ((dcfWeekDay == 7) ? 0 : dcfWeekDay), 0, true);
  // display Weeknumber
  MaximCC.setChar(MaximWeek, 1, ((weekNumber < 10) ? ' ' : (weekNumber / 10)), false);
  MaximCC.setChar(MaximWeek, 0, weekNumber % 10, false);

  // display Date - with dashes between D-M-Y
  // for example: 03-08-2015, so with leading zero's
  MaximCC.setChar(MaximDate, 0, dcfDay / 10, false);
  MaximCC.setChar(MaximDate, 1, dcfDay % 10, false);
  MaximCC.setChar(MaximDate, 2, '-', false);
  MaximCC.setChar(MaximDate, 3, dcfMonth / 10, false);
  MaximCC.setChar(MaximDate, 4, dcfMonth % 10, false);
  MaximCC.setChar(MaximDate, 5  , '-', false);
  MaximCC.setChar(MaximDate, 6, dcfYear / 10, false);
  MaximCC.setChar(MaximDate, 7, dcfYear % 10, false);

/* OTHER OPTIONS TO DISPLAY THE DATE:

  // display Date - with dots between D.M.Y
  // for example: 03.08.15
   MaximCC.setChar(MaximDate, 7, dcfDay / 10, false);
   MaximCC.setChar(MaximDate, 6, dcfDay % 10, true);
   MaximCC.setChar(MaximDate, 5, dcfMonth / 10, false);
   MaximCC.setChar(MaximDate, 4, dcfMonth % 10, true);
   MaximCC.setChar(MaximDate, 3, 2, false);
   MaximCC.setChar(MaximDate, 2, 0, false);
   MaximCC.setChar(MaximDate, 1, dcfYear / 10, false);
   MaximCC.setChar(MaximDate, 0, dcfYear % 10, false);
   */

  /*
  // display Date - moved day to right if month is <10 
  // for example: __3.8.15 or _22.7.15 or 24.12.15 (where _ is a blank display)
   MaximCC.setChar(MaximDate, 7, ((dcfDay < 10) ? ' ' : ((dcfMonth < 10) ? ' ' : (dcfDay / 10))), false);
   MaximCC.setChar(MaximDate, 6, ((dcfMonth < 10) ? ((dcfDay < 10) ? ' ' : (dcfDay / 10)) : (dcfDay % 10)), ((dcfMonth < 10) ? false : true));
   MaximCC.setChar(MaximDate, 5, ((dcfMonth < 10) ? (dcfDay % 10) : (dcfMonth / 10)), ((dcfMonth < 10) ? true : false));
   MaximCC.setChar(MaximDate, 4, dcfMonth % 10, true);
   MaximCC.setChar(MaximDate, 3, 2, false);
   MaximCC.setChar(MaximDate, 2, 0, false);
   MaximCC.setChar(MaximDate, 1, dcfYear / 10, false);
   MaximCC.setChar(MaximDate, 0, dcfYear % 10, false);
   */

  // display Summer- or Wintertime LED
  if (dcfDST == 1) 
  {
    MaximCC.setLed(MaximLeds, SummerTimeLed / 10, SummerTimeLed % 10, true);
    MaximCC.setLed(MaximLeds, WinterTimeLed / 10, WinterTimeLed % 10, false);
  } 
  else {
    MaximCC.setLed(MaximLeds, WinterTimeLed / 10, WinterTimeLed % 10, true);
    MaximCC.setLed(MaximLeds, SummerTimeLed / 10, SummerTimeLed % 10, false);
  }

  // display Leap Year LED
  
  if (leapYear == 1)  
  {
    MaximCC.setLed(MaximLeds, LeapYearLed / 10, LeapYearLed % 10, true);
  } 
  else
  {
    MaximCC.setLed(MaximLeds, LeapYearLed / 10, LeapYearLed % 10, false);
  } 

  

  
}

//================================================================================================================
//
// Function name : displayRtcTime
// called from   : <tasksEverySecond>
//
// Purpose       : display the Real Time Clock time on the RTC display
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void displayRtcTime() 
{
   MaximCC.setChar(MaximRtcTime, 2, 'o',  false); // Black tape mask over "o" displays colon
  MaximCC.setChar(MaximRtcTime, 3, (minute() / 10), false);
  MaximCC.setChar(MaximRtcTime, 4, (minute() % 10), false);
  MaximCC.setChar(MaximRtcTime, 5, 'o', false); // Black tape mask over "o" displays colon
  MaximCC.setChar(MaximRtcTime, 6, (second() / 10), false);
  MaximCC.setChar(MaximRtcTime, 7, (second() % 10), false);

  
  // Brett GMT conversion for RTC
  GMThour = hour() -1 ;
 // Serial.print ("GMT hours ");
 // Serial.println (GMThour);
 // Serial.print ("CET hours ");
 // Serial.println (hour());
 // Serial.println (" ");
  if (GMThour == -1)
  {
    GMThour = 23;
  }
  else
 { 
  GMThour = GMThour;
 }
  MaximCC.setChar(MaximRtcTime, 0, (GMThour / 10), false);
  //lc.setChar(MaximRtcTime, 0, ((GMThour < 10) ? ' ' : (GMThour / 10)), false); // removes leading zero when <10
  MaximCC.setChar(MaximRtcTime, 1, (GMThour % 10), false);

 
}

//================================================================================================================
//
// Function name : calculateTemp
// called from   : <loop>
//
// Purpose       : get temperature from DS18B20 sensor and display it
// Parameters    : none
// Return value  : none
//
//================================================================================================================

//================================================================================================================
//
// Function name : quarterhourchime
// called from   : <tasksEverySecond>
//
// Purpose       : chime 1/4 1/2 & 3/4 hours
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void quarterHourChime()
{
  
  
  if ((minute() == 15 || minute() == 30 || minute() == 45) && second() == 0)
  {
    //load index variable with file index number
    // play selected 1/4 chime through the JQ6500(a IC3)
    switch(minute()){
      case 15:  mp3.playFileByIndexNumber(13); break;
      case 30: mp3.playFileByIndexNumber(14); break;
      case 45: mp3.playFileByIndexNumber(15); break;
     

     
    }
   
  }

if (minute() == 59  && second() == 55)
{
 
      mp3.playFileByIndexNumber(16);
    
    }
   
  
  
   
} //end of quarterHourChime



//================================================================================================================
//
// Function name : hourChime
// called from   : <tasksEverySecond>
//
// Purpose       : chime hours
// Parameters    : none
// Return value  : none
//
//================================================================================================================

void hourChime()
{
   //if its getting near to start of the chime
 
   //if its time for the chime to start 
   
   if (minute() ==  0 && second() == 0)
   {
     //calculate filenumber variable to ensure  results 1 - 12
    if (hour() == 0)// if midnight  make hours = 24
    {
      chimeFile = 24;
    }

     else
     {
     chimeFile = hour() -1;// convert for GMT
     }
     if (chimeFile > 12) chimeFile = chimeFile - 12;
          
     // play selected hour chime through the JQ6500(a IC3)
     switch(chimeFile){
       case 1: mp3.playFileByIndexNumber(1); break;
       case 2: mp3.playFileByIndexNumber(2); break;
       case 3: mp3.playFileByIndexNumber(3); break;
       case 4: mp3.playFileByIndexNumber(4); break;
       case 5: mp3.playFileByIndexNumber(5); break;
       case 6: mp3.playFileByIndexNumber(6); break;
       case 7: mp3.playFileByIndexNumber(7); break;
       case 8: mp3.playFileByIndexNumber(8); break;
       case 9: mp3.playFileByIndexNumber(9); break;
       case 10: mp3.playFileByIndexNumber(10); break;
       case 11: mp3.playFileByIndexNumber(11); break;
       case 12: mp3.playFileByIndexNumber(12); break;
     }
    
   }
} //end of <hourChimes> return to <loop>








//================================================================================================================
//
// Function name : ledTest
// called from   : <setup>
//
// Purpose       : after a cold start, do a led test. 
// Parameters    : none
// Return value  : none
//
//================================================================================================================
void ledTest()
{
  // The displays are lit up sequentially because of the current draw. 
  // When all is lit up at the same time, you would need a bigger power supply. 
  
   //---------------------------------------------------------------------
    // Power up Super filter LEDs 1st then allow 4 seconds for them to self test before doing LED test on Analyzer LEDs
     analogWrite(LEDintensity, 500);  // Brett set super filter LED brightness via pin 9 IC oin 15 to transistor Q1
     analogWrite(LEDintensitya, 500);  // Brett set super filter LED brightness via pin 6 IC pin 12 to transistor Q2
      /*
      MaximCC.setLed(MaximLeds, 0, 1, true);
       MaximCC.setLed(MaximLeds, 1, 1, true);
       MaximCC.setLed(MaximLeds, 2, 1,true);
        MaximCC.setLed(MaximLeds, 3, 1, true);
         MaximCC.setLed(MaximLeds, 4, 1, true);
          MaximCC.setLed(MaximLeds, 5, 1, true);
           MaximCC.setLed(MaximLeds, 6, 1, true);
           MaximCC.setLed(MaximLeds, 7, 1, true);
           
  
             
      delay (4000);
      */
    //  MaximCC.setLed(MaximLeds, 0, 0, false);
    delay (4000);

   
   // Inner LED ring
  for (int i = 0; i < 61; i++) //Brett changed from 59 as 60 LEDs on my clock
  {
    // LED's ON
   // MaximCC.setLed(MaximLedRingInner, i / 8, i % 8, true);
    MaximCC.setLed(MaximLedRingInner, i % 8, i / 8, true);
    delay(LEDTEST_DELAY_LED_RING);
  }
  
  for (int i = 60; i >= 0; i--) //Brett changed from 58 as 60 LEDs on my clock
  {
    // LED's OFF
    //MaximCC.setLed(MaximLedRingInner, i / 8, i % 8, false);
     MaximCC.setLed(MaximLedRingInner, i % 8, i / 8, false);
    delay(LEDTEST_DELAY_LED_RING);
  }
  //---------------------------------------------------------------------
  
  //---------------------------------------------------------------------
  // Outer LED ring
  for (int i = 0; i < 61; i++) ////Brett changed from 59 as 60 LEDs on my clock
  {
    // LED's ON
    //MaximCC.setLed(MaximLedRingOuter, i / 8, i % 8, true);
   MaximCC.setLed(MaximLedRingOuter, i % 8, i / 8, true);
    delay(LEDTEST_DELAY_LED_RING);
  }
  
  for (int i = 60; i >= 0; i--) //Brett changed from 58 as 60 LEDs on my clock
  {
    // LED's OFF
    //MaximCC.setLed(MaximLedRingOuter, i / 8, i % 8, false);
     MaximCC.setLed(MaximLedRingOuter, i % 8, i / 8, false);
    delay(LEDTEST_DELAY_LED_RING);
  }
 
//---------------------------------------------------------------------
  // LED's
 // for (int i = 0; i < 64; i++) 
 for (int i = 0; i < 26; i++) // NOW ONLY 24 ledS CONNECTED
  {
    // LED's ON
   // MaximCC.setLed(MaximLeds, i / 8, i % 8, true);
    MaximCC.setLed(MaximLeds, i % 8, i / 8, true);
    delay(LEDTEST_DELAY_LED_RING2);
    // delay(50); //man delay as LEDs dispersed
  }

  //for (int i = 63; i >= 0; i--)
  for (int i = 26; i >= 0; i--) //NOW ONLY 24 ledS CONNECTED
  {
    // LED's OFF
   // MaximCC.setLed(MaximLeds, i / 8, i % 8, false);
     MaximCC.setLed(MaximLeds, i % 8, i / 8, false);
    delay(LEDTEST_DELAY_LED_RING2);
   //delay(50); //man delay as LEDs dispersed
  }

  //---------------------------------------------------------------------
  // Real Time Clock display
  for (int i = 0; i < 8; i++) 
  {
    // LED's ON
    MaximCC.setChar(MaximRtcTime, i, 8, true);
  }
  // wait before turning the LED's off
  delay(LEDTEST_DELAY_DISPLAYS);
  // clear display
  MaximCC.clearDisplay(MaximRtcTime);
  //---------------------------------------------------------------------
  // Date display
  for (int i = 0; i < 8; i++) 
  {
    // LED's ON
    MaximCC.setChar(MaximDate, i, 8, true);
  }
  // wait before turning the LED's off
  delay(LEDTEST_DELAY_DISPLAYS);
  // clear display
  MaximCC.clearDisplay(MaximDate);
  //---------------------------------------------------------------------
 
//---------------------------------------------------------------------
  // Buffer-DCFbit-Errors display
  for (int i = 0; i < 8; i++) 
  {
    // LED's ON
    MaximCC.setChar(MaximBufferBitError, i, 8, true);
  }
  // wait before turning the LED's off
  delay(LEDTEST_DELAY_DISPLAYS);
  // clear display
  MaximCC.clearDisplay(MaximBufferBitError);

    //---------------------------------------------------------------------
  // Period-Pulse display
  for (int i = 0; i < 8; i++) 
  {
    // LED's ON
    MaximCC.setChar(MaximPeriodPulse, i, 8, true);
  }
  // wait before turning the LED's off
  delay(LEDTEST_DELAY_DISPLAYS);
  // clear display
  MaximCC.clearDisplay(MaximPeriodPulse);
  //---------------------------------------------------------------------
}



//================================================================================================================
//
// Brett check motion detection every second and blank/show displays
//
//================================================================================================================ 

void motiondetection() {
  
 int PIR = digitalRead(13); // reads infrared detector output
   //Serial.print ("PIR value ");
   //Serial.println (PIRval);
 //  Serial.print ("PIR ");
  // Serial.println (PIR);
 // unsigned long currentMillis = millis();
    // check if motion detected
    if (PIR == 1 )
    
      {
        // activate all the displays and status LED's
        for (int i = 0; i < 8; i++)
        {
          MaximCC.shutdown(i, false);  // Common Cathode displays wakeup
        } 
       PIRlock = 1; // Holds on PIR once triggerred
       PIRtimer = 0;
  //     previousMillis = currentMillis;

   //     Serial.println ("PIR On ");
   

      }
    else if  (PIR == 0 && PIRlock == 0)

    {
      // no motion detected  deactivate displays
      
        // deactivate all the displays and status LED's
        for (int i = 0; i < 8; i++)
        {
          MaximCC.shutdown(i, true);  // Common Cathode displays wakeup
        } 
        
// Serial.println ("PIR Off & PIRLock off ");
    } 

    else if  ( PIR == 0 && PIRlock == 1 ) // If PIR triggered and PIRSw has not disabled the PIR 
    {
    
 

 PIRtimer++;
//  Serial.print ("timer ");
//   Serial.println (PIRtimer);  
  if (PIRtimer > 600) // length of time in seconds PIR/Radar detector turns display on before going off.
  // if further movement detected in this time period the timer starts again from 0
{
   PIRlock = 0;
   // Serial.println ("PIRlock off ");
   PIRtimer = 0;
}
   
 } 
    }

 
    




//================================================================================================================
//================================================================================================================
//
// intensity of 7 segment display Brett
//
//================================================================================================================ 


void intensityValue() {  
   
  
  if (ldrValue < 100)
  {
    intensity = 0;
    intensity2 = 0;
    intensity3 = 3;
     intensity4 = 3;
    intensityDisp = 20; // Maxim matrix intenisity 2,0 (3rd row bottom LED lit when intensity = 0 
  }
  else if (ldrValue >= 100 && ldrValue < 150)
  {
    intensity = 1;
    intensity2 = 0;
     intensity3 = 4;
     intensity4 = 4;
    intensityDisp = 21; // Maxim matrix intenisity 2,1 (3rd row 2nd LED lit when intensity = 1
  }


  else if (ldrValue >= 150 && ldrValue < 200)
  {
    intensity = 2;
    intensity2 = 0;
    intensity3 = 4;
     intensity4 = 4;
    intensityDisp = 22;
  }

  else if (ldrValue >= 200 && ldrValue < 250)
  {
    intensity = 3;
    intensity2 = 0;
     intensity3 = 5;
      intensity4 = 5;
    intensityDisp = 23;
  }

  else if (ldrValue >= 250 && ldrValue < 300)
  {
    intensity = 4;
    intensity2 = 0;
     intensity3 = 6;
      intensity4 = 6;
    intensityDisp = 24;
  }


  else if (ldrValue >= 300 && ldrValue < 350)
  {
    intensity = 5;
    intensity2 = 0;
     intensity3 = 6;
      intensity4 = 6;
    intensityDisp = 25;
  }


  else if (ldrValue >= 350 && ldrValue < 400)
  {
    intensity = 6;
    intensity2 = 0;
     intensity3 = 7;
      intensity4 = 7;
    intensityDisp = 26;
  }


  else if (ldrValue >= 400 && ldrValue < 450 )
  {
    intensity = 7;
    intensity2 = 1;
     intensity3 = 8;
      intensity4 = 8;
    intensityDisp = 27;
  }



  else if (ldrValue >= 450 && ldrValue < 500)
  {
    intensity = 8;
    intensity2 = 1;
     intensity3 = 9;
      intensity4 = 9;
    intensityDisp = 30;
  }



  else if (ldrValue >= 500 && ldrValue < 550)
  {
    intensity = 9;
    intensity2 = 1;
     intensity3 = 10;
      intensity4 = 10;
    intensityDisp = 31;
  }



  else if (ldrValue >= 550 && ldrValue < 600)
  {
    intensity = 10;
    intensity2 = 2;
     intensity3 = 11;
      intensity4 =11;
    intensityDisp = 32;
  }



  else if (ldrValue >= 600 && ldrValue < 700)
  {
    intensity = 11;
    intensity2 = 3;
     intensity3 = 12;
    intensityDisp = 33;
  }
  else if (ldrValue >= 700 && ldrValue < 800)
  {
    intensity = 12;
    intensity2 = 4;
     intensity3 = 13;
      intensity4 = 13;
    intensityDisp = 34;
  }
  else if (ldrValue >= 800 && ldrValue < 900)
  {
    intensity = 13;
    intensity2 = 4;
     intensity3 = 14;
      intensity4 = 14;
    intensityDisp = 35;
  }
  else if (ldrValue >= 900 && ldrValue < 999)
  {
    intensity = 14;
    intensity2 = 5;
     intensity3 = 15;
      intensity4 = 15;
    intensityDisp = 36;
  }


  else if (ldrValue >= 999)
  {
    intensity = 15;
    intensity2 = 6;
     intensity3 = 15;
      intensity4 = 15;
    intensityDisp = 37;
  }
  

  
  MaximCC.setIntensity(MaximLedRingInner, BrightnessMaximLedRingInner);
  MaximCC.setIntensity(MaximLedRingOuter, BrightnessMaximLedRingOuter);
  MaximCC.setIntensity(MaximLeds, BrightnessMaximLeds);
  MaximCC.setIntensity(MaximRtcTime, BrightnessMaximRtcTime);
  MaximCC.setIntensity(MaximDate, BrightnessMaximDate);
  MaximCC.setIntensity(MaximBufferBitError, BrightnessMaximBufferBitError);
  MaximCC.setIntensity(MaximPeriodPulse, BrightnessMaximPeriodPulse);
  MaximCC.setIntensity(MaximPeriodPulse, BrightnessMaximPeriodPulse);
  
    
    if (PIRlock == 0)// make sure super filter LEDs blank when not activated by PIR
    {
      analogWrite(LEDintensity, ldrValue);  // Brett set super filter LED brightness via pin 9 IC oin 15 to transistor Q1
     analogWrite(LEDintensitya, ldrValue);  // Brett set super filter LED brightness via pin 6 IC pin 12 to transistor Q2
    }
    
    
    else if (ldrValue < 20)// sets lower limit that super filter LEDs can go to
    {
      analogWrite(LEDintensity, 20); //stops LED intensity going too low at low light values
      analogWrite(LEDintensitya, 20); //stops LED intensity going too low at low light values
    }

    else
    {
    analogWrite(LEDintensity, ldrValue);  // Brett set super filter LED brightness via pin 9 IC oin 15 to transistor Q1
     analogWrite(LEDintensitya, ldrValue);  // Brett set super filter LED brightness via pin 6 IC pin 12 to transistor Q2
  
    }
  /*
      Serial.print("SuperFilter LEDs ldrvalue ");
      Serial.println(ldrValue);

      Serial.print("Large 7 segment intensity ");
      Serial.println(intensity);

      Serial.print("Small 7 segment intensity2 ");
      Serial.println(intensity2);

      Serial.print("Maxim 3 Matrix LEDs intensity3 ");
      Serial.println(intensity3);

      Serial.print("Maxim 1 and 2 inner outer ring  intensity4 ");
      Serial.println(intensity4);
*/

   


 /* 
   // first, clear all the 'Intensity' Led's (row '2 & 3') before displaying new value
  for (int i = 0; i < 8; i++) {
   MaximCC.setLed(MaximLeds, 2, i, false);
   MaximCC.setLed(MaximLeds, 3, i, false);
   // display intensity value LED's
   MaximCC.setLed(MaximLeds, intensityDisp / 10, intensityDisp % 10, true);
   
}
 */ 
} 


//================================================================================================================




